# -*- coding: utf-8 -*-
"""
    hexonet.apiconnector
    ~~~~~~~~~~~~~~~~~~~~
    A connector library for the insanely fast HEXONET Backend API.
    :copyright: © 2018 by HEXONET GmbH.
    :license: MIT, see LICENSE for more details.
"""

__version__ = "3.1.4"
name = "hexonet.apiconnector"



# -*- coding: utf-8 -*-
"""
    hexonet.apiconnector.responsetemplate
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    This module covers all basic functionality to
    deal with Backend API responses.
    :copyright: © 2018 by HEXONET GmbH.
    :license: MIT, see LICENSE for more details.
"""

import hexonet.apiconnector.responseparser as RP


class ResponseTemplate(object):
    """
    The ResponseTemplate class is the base class for the Response Class that
    covers basic functionality to work with Backend API responses.
    """

    def __init__(self, response=""):
        descr = "Empty API response. Probably unreachable API end point"
        #: Holds the response as plain text / string
        self.__raw = response
        if (response is "") or (response is None):
            self.__raw = "[RESPONSE]\r\nCODE=423\r\nDESCRIPTION=%s\r\nEOF\r\n" % (descr)

        # try/except to support old versions of python (python2.5)
        try:
            if isinstance(self.__raw, bytes):
                self.__raw = self.__raw.decode("utf-8")
        except UnicodeError:
            self.__raw = self.__raw.decode("latin1")
        except BaseException:
            self.__raw = self.__raw.decode("utf-8")

        if isinstance(response, dict):
            raise TypeError('Type "dict" is not allowed for parameter "response". Use type "string" instead.')
        else:
            #: Holds the response as hash
            self.__hash = RP.parse(self.__raw)

    def getCode(self):
        """
        Returns the API response code as integer
        """
        return int(self.__hash['CODE'])

    def getDescription(self):
        """
        Returns the API response description
        """
        return self.__hash['DESCRIPTION']

    def getPlain(self):
        """
        Returns the plain API response
        """
        return self.__raw

    def getQueuetime(self):
        """
        Get Queuetime of API response as float value
        """
        if ("QUEUETIME" in self.__hash):
            return float(self.__hash["QUEUETIME"])
        return 0.00

    def getHash(self):
        """
        Get API response as Hash
        """
        return self.__hash

    def getRuntime(self):
        """
        Get Runtime of API response as float value
        """
        if ("RUNTIME" in self.__hash):
            return float(self.__hash["RUNTIME"])
        return 0.00

    def isError(self):
        """
        Check if current API response represents an error case (5xx)
        """
        return self.__hash["CODE"][0] == "5"

    def isSuccess(self):
        """
        Check if current API response represents a success case (2xx)
        """
        return self.__hash["CODE"][0] == "2"

    def isTmpError(self):
        """
        Check if current API response represents a temporary error case (4xx)
        """
        return self.__hash["CODE"][0] == "4"

    def isPending(self):
        """
        Check if current operation is returned as pending
        """
        if 'PENDING' in self.__hash:
            return self.__hash["PENDING"] == "1"
        return False



# -*- coding: utf-8 -*-
"""
    hexonet.apiconnector.responseparser
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    This module covers all necessary functionality to
    parse a raw Backend API response and to serialize it back.
    :copyright: © 2018 by HEXONET GmbH.
    :license: MIT, see LICENSE for more details.
"""

import re


def parse(raw):
    """
      Returns the response as a string
    """
    r = {}
    re1 = re.compile(r'^([^\=]*[^\t\= ])[\t ]*=[\t ]*(.*)$')
    re2 = re.compile(r'^property\[([^\]]*)\]', re.IGNORECASE)

    raw = raw.replace('\r\n', '\n')
    for line in raw.split("\n"):
        m1 = re1.match(line)
        if not m1:
            continue
        attr = m1.group(1)
        value = m1.group(2)
        value = re.sub(r'[\t ]*$', '', value)

        m2 = re2.match(attr)
        if m2:
            prop = m2.group(1).upper()
            prop = re.sub(r'\s', '', prop)
            if ("PROPERTY" not in r):
                r["PROPERTY"] = {}
            if prop not in r['PROPERTY']:
                r['PROPERTY'][prop] = []
            r['PROPERTY'][prop].append(value)
        else:
            r[attr.upper()] = value

    if 'DESCRIPTION' not in r:
        r['DESCRIPTION'] = ""
    return r


def serialize(r):
    """
      Returns the dictionary represenation serialized back to plain text
    """
    d = ""
    if ("PROPERTY" in r):
        keys = sorted(r['PROPERTY'].keys())
        for key in keys:
            for index, val in enumerate(r['PROPERTY'][key]):
                d += ("\r\nPROPERTY[{0}][{1}]={2}").format(key, index, val)
    for prop in ["CODE", "DESCRIPTION", "QUEUETIME", "RUNTIME"]:
        if (prop in r):
            d += ('\r\n{0}={1}').format(prop, r[prop])
    return ("[RESPONSE]{0}\r\nEOF\r\n").format(d)



#!/usr/bin/env python

import os
import io
import re
from setuptools import setup, find_packages


def read(*names, **kwargs):
    with io.open(
        os.path.join(os.path.dirname(__file__), *names),
        encoding=kwargs.get("encoding", "utf8")
    ) as fp:
        return fp.read()


def find_version(*file_paths):
    version_file = read(*file_paths)
    version_match = re.search(r"^__version__ = ['\"]([^'\"]*)['\"]",
                              version_file, re.M)
    if version_match:
        return version_match.group(1)
    raise RuntimeError("Unable to find version string.")


with open("README.md", "r") as fh:
    long_description = fh.read()

PACKAGE = 'hexonet.apiconnector'
VERSION = find_version('hexonet', 'apiconnector', '__init__.py')

setup(
    name=PACKAGE,
    version=VERSION,
    description=PACKAGE + ' is a connector library for the insanely fast HEXONET Backend API',
    long_description=long_description,
    long_description_content_type="text/markdown",
    author='Anthony Schneider',
    author_email='anthonys@hexonet.net',
    maintainer='Kai Schwarz',
    maintainer_email='kschwarz@hexonet.net',
    url='https://github.com/hexonet/python-sdk/',
    install_require=[
        'autopep8',
        'guzzle_sphinx_theme',
        'pycodestyle',
        'sphinx',
        'sphinxcontrib-log-cabinet',
        'twine'
    ],
    setup_requires=[
        'autopep8',
        'guzzle_sphinx_theme',
        'pycodestyle',
        'sphinx',
        'sphinxcontrib-log-cabinet',
        'twine'
    ],
    tests_require=['six'],
    license="MIT",
    scripts=[],
    zip_safe=True,
    classifiers=(
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent"
    ),
    namespace_packages=['hexonet'],
    packages=find_packages()
)
